@@ -118,12 +118,12 @@ static int dec_yuv_buffers = IVTV_DEFAULT_DEC_YUV_BUFFERS;
 static int dec_vbi_buffers = IVTV_DEFAULT_DEC_VBI_BUFFERS;
 
 static int ivtv_yuv_mode = 0;
-static int ivtv_yuv_threshold=480;
+static int ivtv_yuv_threshold=-1;
 static int ivtv_pci_latency = 1;
 
-int ivtv_debug = IVTV_DBGFLG_WARN;
+int ivtv_debug = 0;
 
-int newi2c = -1;
+static int newi2c = -1;
 
 module_param_array(tuner, int, &tuner_c, 0644);
 module_param_array(radio, bool, &radio_c, 0644);
@@ -329,6 +329,7 @@ static void ivtv_process_eeprom(struct ivtv *itv)
 		// In a few cases the PCI subsystem IDs do not correctly
 		// identify the card. A better method is to check the
 		// model number from the eeprom instead.
+		case 30012 ... 30039:  /* Low profile PVR250 */
                 case 32000 ... 32999:
                 case 48000 ... 48099:  /* 48??? range are PVR250s with a cx23415 */
                 case 48400 ... 48599:
@@ -615,31 +616,23 @@ static int __devinit ivtv_init_struct1(struct ivtv *itv)
 	/* Set FrameBuffer-ID to invalid */
 	itv->fb_id = -1;
 
+	mutex_init(&itv->serialize_lock);
 	mutex_init(&itv->i2c_bus_lock);
 	mutex_init(&itv->udma.lock);
 
 	itv->lock = SPIN_LOCK_UNLOCKED;
 	itv->dma_reg_lock = SPIN_LOCK_UNLOCKED;
 
-	itv->vbi.work_queues = create_workqueue("ivtv_vbi");
-	if (itv->vbi.work_queues == NULL) {
-		IVTV_ERR("Could not create VBI workqueue\n");
-		return -1;
-	}
-
-	itv->yuv_info.work_queues = create_workqueue("ivtv_yuv");
-	if (itv->yuv_info.work_queues == NULL) {
-		IVTV_ERR("Could not create YUV workqueue\n");
-		destroy_workqueue(itv->vbi.work_queues);
+	itv->irq_work_queues = create_workqueue(itv->name);
+	if (itv->irq_work_queues == NULL) {
+		IVTV_ERR("Could not create ivtv workqueue\n");
 		return -1;
 	}
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
-	INIT_WORK(&itv->vbi.work_queue, vbi_work_handler);
-	INIT_WORK(&itv->yuv_info.work_queue, ivtv_yuv_work_handler);
+	INIT_WORK(&itv->irq_work_queue, ivtv_irq_work_handler);
 #else
-	INIT_WORK(&itv->vbi.work_queue, vbi_work_handler, itv);
-	INIT_WORK(&itv->yuv_info.work_queue, ivtv_yuv_work_handler, itv);
+	INIT_WORK(&itv->irq_work_queue, ivtv_irq_work_handler, itv);
 #endif
 
 	/* start counting open_id at 1 */
@@ -657,6 +650,7 @@ static int __devinit ivtv_init_struct1(struct ivtv *itv)
 	itv->dma_timer.data = (unsigned long)itv;
 
 	itv->cur_dma_stream = -1;
+	itv->cur_pio_stream = -1;
 
 	/* Ctrls */
 	itv->dec_options.speed.mute = 1;
@@ -1156,7 +1150,7 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 
 	/* Register IRQ */
 	retval = request_irq(itv->dev->irq, ivtv_irq_handler,
-			     SA_SHIRQ | SA_INTERRUPT, itv->name, (void *)itv);
+			     IRQF_SHARED | IRQF_DISABLED, itv->name, (void *)itv);
 	if (retval) {
 		IVTV_ERR("Failed to register irq %d\n", retval);
 		goto free_streams;
@@ -1232,8 +1226,7 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 	if (itv->has_cx23415)
 		release_mem_region(itv->base_addr + IVTV_DECODER_OFFSET, IVTV_DECODER_SIZE);
       free_workqueue:
-	destroy_workqueue(itv->vbi.work_queues);
-	destroy_workqueue(itv->yuv_info.work_queues);
+	destroy_workqueue(itv->irq_work_queues);
       err:
 	if (retval == 0)
 		retval = -ENODEV;
@@ -1275,10 +1268,8 @@ static void ivtv_remove(struct pci_dev *pci_dev)
 
 	/* Stop all Work Queues */
 	IVTV_DEBUG_INFO(" Stop Work Queues.\n");
-	flush_workqueue(itv->vbi.work_queues);
-	flush_workqueue(itv->yuv_info.work_queues);
-	destroy_workqueue(itv->vbi.work_queues);
-	destroy_workqueue(itv->yuv_info.work_queues);
+	flush_workqueue(itv->irq_work_queues);
+	destroy_workqueue(itv->irq_work_queues);
 
 	IVTV_DEBUG_INFO(" Stopping Firmware.\n");
 	ivtv_halt_firmware(itv);
@@ -1334,9 +1325,9 @@ static int module_start(void)
 		return -1;
 	}
 
-	if (ivtv_debug < 0 || ivtv_debug > 511) {
-		ivtv_debug = 1;
-		printk(KERN_INFO "ivtv:  debug value must be >= 0 and <= 511!\n");
+	if (ivtv_debug < 0 || ivtv_debug > 1023) {
+		ivtv_debug = 0;
+		printk(KERN_INFO "ivtv:  debug value must be >= 0 and <= 1023!\n");
 	}
 
 	if (pci_module_init(&ivtv_pci_driver)) {
@@ -1351,6 +1342,8 @@ static void module_cleanup(void)
 {
         int i, j;
 
+	pci_unregister_driver(&ivtv_pci_driver);
+
         for (i = 0; i < ivtv_cards_active; i++) {
 		if (ivtv_cards[i] == NULL)
 			continue;
@@ -1359,7 +1352,6 @@ static void module_cleanup(void)
                 }
                 kfree(ivtv_cards[i]);
         }
-	pci_unregister_driver(&ivtv_pci_driver);
 }
 
 EXPORT_SYMBOL(ivtv_set_irq_mask);
