@@ -243,6 +243,7 @@ extern const u32 yuv_offset[4];
 #define IVTV_IRQ_ENC_VBI_CAP		(0x1 << 29)
 #define IVTV_IRQ_ENC_VIM_RST		(0x1 << 28)
 #define IVTV_IRQ_ENC_DMA_COMPLETE	(0x1 << 27)
+#define IVTV_IRQ_ENC_PIO_COMPLETE	(0x1 << 25)
 #define IVTV_IRQ_DEC_AUD_MODE_CHG	(0x1 << 24)
 #define IVTV_IRQ_DEC_DATA_REQ		(0x1 << 22)
 #define IVTV_IRQ_DEC_DMA_COMPLETE	(0x1 << 20)
@@ -253,7 +254,8 @@ extern const u32 yuv_offset[4];
 #define IVTV_IRQ_DEC_VSYNC		(0x1 << 10)
 
 /* IRQ Masks */
-#define IVTV_IRQ_MASK_INIT (IVTV_IRQ_DMA_ERR|IVTV_IRQ_ENC_DMA_COMPLETE|IVTV_IRQ_DMA_READ)
+#define IVTV_IRQ_MASK_INIT (IVTV_IRQ_DMA_ERR|IVTV_IRQ_ENC_DMA_COMPLETE|\
+		IVTV_IRQ_DMA_READ|IVTV_IRQ_ENC_PIO_COMPLETE)
 
 #define IVTV_IRQ_MASK_CAPTURE (IVTV_IRQ_ENC_START_CAP | IVTV_IRQ_ENC_EOS)
 #define IVTV_IRQ_MASK_DECODE  (IVTV_IRQ_DEC_DATA_REQ|IVTV_IRQ_DEC_AUD_MODE_CHG)
@@ -279,6 +281,21 @@ extern const u32 yuv_offset[4];
 #define IVTV_DEBUG_DEC(fmt, args...)   IVTV_DEBUG(IVTV_DBGFLG_DEC, "dec", fmt , ## args)
 #define IVTV_DEBUG_YUV(fmt, args...)   IVTV_DEBUG(IVTV_DBGFLG_YUV, "yuv", fmt , ## args)
 
+#define IVTV_DEBUG_HIGH_VOL(x, type, fmt, args...) \
+	do { \
+		if (((x) & ivtv_debug) && (ivtv_debug & IVTV_DBGFLG_HIGHVOL)) \
+               		printk(KERN_INFO "ivtv%d " type ": " fmt, itv->num , ## args); \
+	} while (0)
+#define IVTV_DEBUG_HI_WARN(fmt, args...)  IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_WARN, "warning", fmt , ## args)
+#define IVTV_DEBUG_HI_INFO(fmt, args...)  IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_INFO, "info",fmt , ## args)
+#define IVTV_DEBUG_HI_API(fmt, args...)   IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_API, "api", fmt , ## args)
+#define IVTV_DEBUG_HI_DMA(fmt, args...)   IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_DMA, "dma", fmt , ## args)
+#define IVTV_DEBUG_HI_IOCTL(fmt, args...) IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_IOCTL, "ioctl", fmt , ## args)
+#define IVTV_DEBUG_HI_I2C(fmt, args...)   IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_I2C, "i2c", fmt , ## args)
+#define IVTV_DEBUG_HI_IRQ(fmt, args...)   IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_IRQ, "irq", fmt , ## args)
+#define IVTV_DEBUG_HI_DEC(fmt, args...)   IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_DEC, "dec", fmt , ## args)
+#define IVTV_DEBUG_HI_YUV(fmt, args...)   IVTV_DEBUG_HIGH_VOL(IVTV_DBGFLG_YUV, "yuv", fmt , ## args)
+
 #define IVTV_FB_DEBUG(x, type, fmt, args...) \
 	do { \
 		if ((x) & ivtv_debug) \
@@ -374,21 +391,28 @@ struct ivtv_mailbox_data {
 #define IVTV_F_S_STREAMOFF	7	/* signal end of stream EOS */
 #define IVTV_F_S_APPL_IO        8	/* this stream is used read/written by an application */
 
-/* per-ivtv, i_flags */
-#define IVTV_F_I_DMA	        0 	/* DMA in progress */
-#define IVTV_F_I_UDMA	        1 	/* UDMA in progress */
-#define IVTV_F_I_UDMA_PENDING	2 	/* UDMA pending */
+#define IVTV_F_S_PIO_PENDING	9	/* this stream has pending PIO */
+#define IVTV_F_S_PIO_HAS_VBI	1       /* the current PIO request also requests VBI data */
 
-#define IVTV_F_I_SPEED_CHANGE	3 	/* A speed change is in progress */
-#define IVTV_F_I_EOS		4 	/* End of encoder stream reached */
-#define IVTV_F_I_RADIO_USER	5 	/* The radio tuner is selected */
-#define IVTV_F_I_DIG_RST	6 	/* Reset digitizer */
-#define IVTV_F_I_DEC_YUV	7 	/* YUV instead of MPG is being decoded */
-#define IVTV_F_I_ENC_VBI	8 	/* VBI DMA */
-#define IVTV_F_I_UPDATE_CC	9  	/* CC should be updated */
-#define IVTV_F_I_UPDATE_WSS	10 	/* WSS should be updated */
-#define IVTV_F_I_UPDATE_VPS	11 	/* VPS should be updated */
-#define IVTV_F_I_DECODING_YUV	12 	/* this stream is YUV frame decoding */
+/* per-ivtv, i_flags */
+#define IVTV_F_I_DMA	          0 	/* DMA in progress */
+#define IVTV_F_I_UDMA	          1 	/* UDMA in progress */
+#define IVTV_F_I_UDMA_PENDING	  2 	/* UDMA pending */
+#define IVTV_F_I_SPEED_CHANGE	  3 	/* A speed change is in progress */
+#define IVTV_F_I_EOS		  4 	/* End of encoder stream reached */
+#define IVTV_F_I_RADIO_USER	  5 	/* The radio tuner is selected */
+#define IVTV_F_I_DIG_RST	  6 	/* Reset digitizer */
+#define IVTV_F_I_DEC_YUV	  7 	/* YUV instead of MPG is being decoded */
+#define IVTV_F_I_ENC_VBI	  8 	/* VBI DMA */
+#define IVTV_F_I_UPDATE_CC	  9  	/* CC should be updated */
+#define IVTV_F_I_UPDATE_WSS	  10 	/* WSS should be updated */
+#define IVTV_F_I_UPDATE_VPS	  11 	/* VPS should be updated */
+#define IVTV_F_I_DECODING_YUV	  12 	/* this stream is YUV frame decoding */
+#define IVTV_F_I_HAVE_WORK  	   15	/* Used in the interrupt handler: there is work to be done */
+#define IVTV_F_I_WORK_HANDLER_VBI  16	/* there is work to be done for VBI */
+#define IVTV_F_I_WORK_HANDLER_YUV  17	/* there is work to be done for YUV */
+#define IVTV_F_I_WORK_HANDLER_PIO  18	/* there is work to be done for PIO */
+#define IVTV_F_I_PIO		   19	/* PIO in progress */
 
 /* Scatter-Gather array element, used in DMA transfers */
 struct ivtv_SG_element {
@@ -475,6 +499,7 @@ struct ivtv_stream {
 
 	/* Base Dev SG Array for cx23415/6 */
 	struct ivtv_SG_element *SGarray;
+	struct ivtv_SG_element *PIOarray;
 	dma_addr_t SG_handle;
 	int SG_length;
 
@@ -579,7 +604,6 @@ struct yuv_playback_info
 
 	int lace_mode;
 	int lace_threshold;
-	int lace_threshold_last;
 	int lace_sync_field;
 
 	atomic_t next_dma_frame;
@@ -587,8 +611,6 @@ struct yuv_playback_info
 
 	u32 yuv_forced_update;
 	int update_frame;
-	struct workqueue_struct *work_queues;
-	struct work_struct work_queue;
 	struct yuv_frame_info new_frame_info[4];
 	struct yuv_frame_info old_frame_info;
 	struct yuv_frame_info old_frame_info_args;
@@ -625,7 +647,6 @@ struct vbi_info {
        	/* convenience pointer to sliced struct in vbi_in union */
         struct v4l2_sliced_vbi_format *sliced_in;
 	u32 service_set_in;
-	u32 service_set_out;
 	int insert_mpeg;
 
 	/* Buffer for the maximum of 2 * 18 * packet_size sliced VBI lines.
@@ -651,8 +672,6 @@ struct vbi_info {
 	struct ivtv_buffer sliced_mpeg_buf;
 	u32 inserted_frame;
 
-	struct workqueue_struct *work_queues;
-	struct work_struct work_queue;
 	u32 start[2], count;
 	u32 raw_size;
 	u32 sliced_size;
@@ -689,6 +708,7 @@ struct ivtv {
 	atomic_t decoding;	/* count number of active decoding streams */
 	u32 irq_rr_idx; /* Round-robin stream index */
 	int cur_dma_stream;	/* index of stream doing DMA */
+	int cur_pio_stream;	/* index of stream doing PIO */
 	u32 dma_data_req_offset;
 	u32 dma_data_req_size;
         int output_mode;        /* NONE, MPG, YUV, UDMA YUV, passthrough */
@@ -696,6 +716,7 @@ struct ivtv {
 	int search_pack_header;
 
 	spinlock_t dma_reg_lock; /* lock access to DMA engine registers */
+	struct mutex serialize_lock;  /* lock used to serialize starting streams */
 	
 	/* User based DMA for OSD */
 	struct ivtv_user_dma udma;
@@ -706,6 +727,9 @@ struct ivtv {
 
 	u32 base_addr;
 	u32 irqmask;
+
+	struct workqueue_struct *irq_work_queues;
+	struct work_struct irq_work_queue;
 	struct timer_list dma_timer; /* Timer used to catch unfinished DMAs */
 
         struct vbi_info vbi;
