@@ -234,7 +234,7 @@ static int ivtv_ack(struct ivtv *itv)
 	int ret = 0;
 	
 	if (ivtv_getscl(itv) == 1) {
-		IVTV_DEBUG_I2C("SCL was high starting an ack\n");
+		IVTV_DEBUG_HI_I2C("SCL was high starting an ack\n");
 		ivtv_setscl(itv, 0);
 		if (!ivtv_waitscl(itv, 0)) {
 			IVTV_DEBUG_I2C("Could not set SCL low starting an ack\n");
@@ -261,7 +261,7 @@ static int ivtv_sendbyte(struct ivtv *itv, unsigned char byte)
 {
 	int i, bit;
 	
-	IVTV_DEBUG_I2C("write %x\n",byte);
+	IVTV_DEBUG_HI_I2C("write %x\n",byte);
 	for (i = 0; i < 8; ++i, byte<<=1) {
 		ivtv_setscl(itv, 0);
 		if (!ivtv_waitscl(itv, 0)) {
@@ -316,7 +316,7 @@ static int ivtv_readbyte(struct ivtv *itv, unsigned char *byte, int nack)
 	ivtv_scldelay(itv);
 	ivtv_setscl(itv, 0);
 	ivtv_scldelay(itv);
-	IVTV_DEBUG_I2C("read %x\n",*byte);
+	IVTV_DEBUG_HI_I2C("read %x\n",*byte);
 	return 0;
 }
 
@@ -328,7 +328,7 @@ static int ivtv_start(struct ivtv *itv)
 	
 	sda = ivtv_getsda(itv);
 	if (sda != 1) {
-		IVTV_DEBUG_I2C("SDA was low at start\n");
+		IVTV_DEBUG_HI_I2C("SDA was low at start\n");
 		ivtv_setsda(itv, 1);
 		if (!ivtv_waitsda(itv, 1)) {
 			IVTV_DEBUG_I2C("SDA stuck low\n");
@@ -353,7 +353,7 @@ static int ivtv_stop(struct ivtv *itv)
 	int i;
 	
 	if (ivtv_getscl(itv) != 0) {
-		IVTV_DEBUG_I2C("SCL not low when stopping\n");
+		IVTV_DEBUG_HI_I2C("SCL not low when stopping\n");
 		ivtv_setscl(itv, 0);
 		if (!ivtv_waitscl(itv, 0)) {
 			IVTV_DEBUG_I2C("SCL could not be set low\n");
@@ -576,7 +576,7 @@ static int ivtv_call_i2c_client(struct ivtv *itv, int addr, unsigned int cmd,
 
 /* Find the i2c device based on the driver ID and return
    its i2c address or -ENODEV if no matching device was found. */
-int ivtv_i2c_id_addr(struct ivtv *itv, u32 id)
+static int ivtv_i2c_id_addr(struct ivtv *itv, u32 id)
 {
 	struct i2c_client *client;
 	int retval = -ENODEV;
@@ -682,11 +682,6 @@ int ivtv_saa717x(struct ivtv *itv, unsigned int cmd, void *arg)
 	return ivtv_call_i2c_client(itv, IVTV_SAA717x_I2C_ADDR, cmd, arg);
 }
 
-int ivtv_msp34xx(struct ivtv *itv, unsigned int cmd, void *arg)
-{
-	return ivtv_call_i2c_client(itv, IVTV_MSP3400_I2C_ADDR, cmd, arg);
-}
-
 int ivtv_upd64031a(struct ivtv *itv, unsigned int cmd, void *arg)
 {
 	return ivtv_call_i2c_client(itv, IVTV_UPD64031A_I2C_ADDR, cmd, arg);
