@@ -18,6 +18,16 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs);
-void ivtv_sched_DMA(struct ivtv *itv);
-void ivtv_sched_VBI(struct ivtv *itv, int streamtype);
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 18)
+irqreturn_t ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs);
+#else
+irqreturn_t ivtv_irq_handler(int irq, void *dev_id);
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
+void ivtv_irq_work_handler(struct work_struct *work);
+#else
+void ivtv_irq_work_handler(void *arg);
+#endif
+void ivtv_dma_stream_dec_prepare(struct ivtv_stream *s, u32 offset, int lock);
+void ivtv_unfinished_dma(unsigned long arg);
