@@ -66,7 +66,7 @@ static struct file_operations ivtv_v4l2_dec_fops = {
       .poll = ivtv_v4l2_dec_poll,
 };
 
-struct {
+static struct {
 	const char *name;
 	int vfl_type;
 	int minor_offset;
@@ -446,6 +446,9 @@ int ivtv_start_v4l2_encode_stream(struct ivtv_stream *s)
 	if (s->v4l2dev == NULL)
 		return -EINVAL;
 
+        /* Big serialization lock to ensure no two streams are started
+           simultaneously: that can give all sorts of weird results. */
+        mutex_lock(&itv->serialize_lock);
 	IVTV_DEBUG_INFO("Start encoder stream %s\n", s->name);
 
 	switch (s->type) {
@@ -487,6 +490,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv_stream *s)
                         0, sizeof(itv->vbi.sliced_mpeg_size));
 		break;
 	default:
+                mutex_unlock(&itv->serialize_lock);
 		return -EINVAL;
 	}
 	s->subtype = subtype;
@@ -562,6 +566,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv_stream *s)
 	if (ivtv_vapi(itv, CX2341X_ENC_START_CAPTURE, 2, captype, subtype)) 
 	{
 		IVTV_DEBUG_WARN( "Error starting capture!\n");
+                mutex_unlock(&itv->serialize_lock);
 		return -EINVAL;
 	}
 
@@ -577,6 +582,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv_stream *s)
 
 	/* you're live! sit back and await interrupts :) */
 	atomic_inc(&itv->capturing);
+        mutex_unlock(&itv->serialize_lock);
 	return 0;
 }
 
@@ -759,17 +765,6 @@ int ivtv_stop_capture(struct ivtv_stream *s)
 	/* when: 0 =  end of GOP  1 = NOW!, type: 0 = mpeg, subtype: 3 = video+audio */
 	ivtv_vapi(itv, CX2341X_ENC_STOP_CAPTURE, 3, stopmode, cap_type, s->subtype);
 
-	/* only run these if we're shutting down the last cap */
-	if (atomic_read(&itv->capturing) - 1 == 0) {
-		/* event notification (off) */
-		if (test_and_clear_bit(IVTV_F_I_DIG_RST, &itv->i_flags)) {
-			/* type: 0 = refresh */
-			/* on/off: 0 = off, intr: 0x10000000, mbox_id: -1: none */
-			ivtv_vapi(itv, CX2341X_ENC_SET_EVENT_NOTIFICATION, 4, 0, 0, IVTV_IRQ_ENC_VIM_RST, -1);
-			ivtv_set_irq_mask(itv, IVTV_IRQ_ENC_VIM_RST);
-		}
-	}
-
 	then = jiffies;
 
 	if (!test_bit(IVTV_F_S_PASSTHROUGH, &s->s_flags)) {
@@ -809,7 +804,6 @@ int ivtv_stop_capture(struct ivtv_stream *s)
 		then = jiffies;
 		/* Make sure DMA is complete */
 		add_wait_queue(&s->waitq, &wait);
-		set_current_state(TASK_INTERRUPTIBLE);
 		do {
 			/* check if DMA is pending */
 			if ((s->type == IVTV_ENC_STREAM_TYPE_MPG) &&	/* MPG Only */
@@ -824,9 +818,7 @@ int ivtv_stop_capture(struct ivtv_stream *s)
 			} else if (read_reg(IVTV_REG_DMASTATUS) & 0x02) {
 				break;
 			}
-
-			ivtv_sleep_timeout(HZ / 100, 1);
-		} while (then + HZ * 2 > jiffies);
+		} while (!ivtv_sleep_timeout(HZ / 100, 1) && then + HZ * 2 > jiffies);
 
 		set_current_state(TASK_RUNNING);
 		remove_wait_queue(&s->waitq, &wait);
@@ -837,17 +829,30 @@ int ivtv_stop_capture(struct ivtv_stream *s)
 	/* Clear capture and no-read bits */
 	clear_bit(IVTV_F_S_STREAMING, &s->s_flags);
 
+        /* ensure these global cleanup actions are done only once */
+        mutex_lock(&itv->serialize_lock);
+
         if (s->type == IVTV_ENC_STREAM_TYPE_VBI)
                 ivtv_set_irq_mask(itv, IVTV_IRQ_ENC_VBI_CAP);
 
 	if (atomic_read(&itv->capturing) > 0) {
+                mutex_unlock(&itv->serialize_lock);
 		return 0;
 	}
 
 	/* Set the following Interrupt mask bits for capture */
 	ivtv_set_irq_mask(itv, IVTV_IRQ_MASK_CAPTURE);
 
+        /* event notification (off) */
+        if (test_and_clear_bit(IVTV_F_I_DIG_RST, &itv->i_flags)) {
+                /* type: 0 = refresh */
+                /* on/off: 0 = off, intr: 0x10000000, mbox_id: -1: none */
+                ivtv_vapi(itv, CX2341X_ENC_SET_EVENT_NOTIFICATION, 4, 0, 0, IVTV_IRQ_ENC_VIM_RST, -1);
+                ivtv_set_irq_mask(itv, IVTV_IRQ_ENC_VIM_RST);
+        }
+
 	wake_up(&s->waitq);
+        mutex_unlock(&itv->serialize_lock);
 
 	return 0;
 }
