@@ -66,6 +66,11 @@ MODULE_DESCRIPTION("Philips SAA717x video decoder driver");
 MODULE_AUTHOR("Kevin Thayer, K. Ohta");
 MODULE_LICENSE("GPL");
 
+#ifdef VIDIOC_DBG_S_REGISTER
+#define VIDIOC_INT_S_REGISTER VIDIOC_DBG_S_REGISTER
+#define VIDIOC_INT_G_REGISTER VIDIOC_DBG_G_REGISTER
+#endif
+
 static int debug = 0;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 module_param(debug, int, 0644);
@@ -145,7 +150,7 @@ static int saa717x_write(struct i2c_client *client, u32 reg, u32 value)
 {
 	struct i2c_adapter *adap = client->adapter;
 	//int fw_addr = (reg >= 0x404 && reg <= 0x4b8);  // Fix Below 'cos register area is more limited.
-   	int fw_addr = ((reg == 0x454) || ((reg >= 0x464) && (reg <= 0x478)));
+   	int fw_addr = ((reg == 0x454) || ((reg >= 0x464) && (reg <= 0x478)) || (reg == 0x480) || (reg == 0x488));
 	unsigned char mm1[6];
 	struct i2c_msg msg;
 
@@ -1257,7 +1262,11 @@ static int saa717x_command(struct i2c_client *client, unsigned int cmd, void *ar
 	{
 		struct v4l2_register *reg = arg;
 
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,20)
 		if (reg->i2c_id != I2C_DRIVERID_SAA717X)
+#else
+		if (!v4l2_chip_match_i2c_client(client, reg->match_type, reg->match_chip))
+#endif
 			return -EINVAL;
 		reg->val = saa717x_read(client, reg->reg);
 		break;
@@ -1269,7 +1278,11 @@ static int saa717x_command(struct i2c_client *client, unsigned int cmd, void *ar
 		u16 addr = reg->reg & 0xffff;
 		u8 val = reg->val & 0xff;
 
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,20)
 		if (reg->i2c_id != I2C_DRIVERID_SAA717X)
+#else
+		if (!v4l2_chip_match_i2c_client(client, reg->match_type, reg->match_chip))
+#endif
 			return -EINVAL;
 		if (!capable(CAP_SYS_ADMIN))
 			return -EPERM;
@@ -1539,13 +1552,14 @@ static int saa717x_detect_client (struct i2c_adapter *adapter,
 	    saa717x_write(client, 0x5a7, 0x01)) {
 		id = saa717x_read(client, 0x5a0);
 	}
-	if (id != 0xc2 && id != 0x32 && id != 0xf2) {
+	if (id != 0xc2 && id != 0x32 && id != 0xf2 && id != 0x6c) {
 		v4l_dbg(1, debug, client, "saa717x not found (id=%02x)\n", id);
 		kfree(client);
 		return 0;
 	}
 	if (id == 0xc2) p = "saa7173";
 	else if (id == 0x32) p = "saa7174A";
+	else if (id == 0x6c) p = "saa7174HL";
 	else p = "saa7171";
 	v4l_info(client, "%s found @ 0x%x (%s)\n", p, address << 1, adapter->name);
 
