@@ -35,6 +35,7 @@
 #include <sys/time.h>
 #include <math.h>
 #include <linux/types.h>
+#include <linux/version.h>
 #include <sys/klog.h>
 
 #include "videodev2.h"
@@ -46,6 +47,20 @@ struct v4l2_routing {
 };
 #define	VIDIOC_INT_S_AUDIO_ROUTING	_IOW ('d', 109, struct v4l2_routing)
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
+#define V4L2_CHIP_MATCH_HOST       0  /* Match against chip ID on host (0 for the host) */
+#define V4L2_CHIP_MATCH_I2C_DRIVER 1  /* Match against I2C driver ID */
+#define V4L2_CHIP_MATCH_I2C_ADDR   2  /* Match against I2C 7-bit address */
+
+struct v4l2_register {
+	__u32 match_type; /* Match type */
+	__u32 i2c_id; /* Match this chip, meaning determined by match_type */
+	__u64 reg;
+	__u64 val;
+};
+#define	VIDIOC_INT_S_REGISTER 	_IOW  ('V', 79, struct v4l2_register)
+#define	VIDIOC_INT_G_REGISTER 	_IOWR ('V', 80, struct v4l2_register)
+#else
 struct v4l2_register {
 	__u32 i2c_id; 		/* I2C driver ID of the I2C chip. 0 for the I2C adapter. */
 	unsigned long reg;
@@ -53,7 +68,12 @@ struct v4l2_register {
 };
 #define	VIDIOC_INT_S_REGISTER 		_IOW ('d', 100, struct v4l2_register)
 #define	VIDIOC_INT_G_REGISTER 		_IOWR('d', 101, struct v4l2_register)
+#endif
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)
+#define VIDIOC_INT_RESET            	_IOW ('d', 102, __u32)
+#else
 #define VIDIOC_INT_RESET            	_IO  ('d', 102)
+#endif
 
 #define IVTV_INTERNAL
 #include "ivtv.h"
@@ -143,8 +163,8 @@ static void usage(void)
 	printf("Expert options:\n");
 	printf("  -D, --set-debug <level>\n");
 	printf("                     set the module debug variable\n");
-	printf("                      1: WARN   2: INFO  4: API   8: DMA\n");
-	printf("                     16: IOCTL 32: I2C  64: IRQ 128: DEC 256: YUV\n");
+	printf("                      1: WARN   2: INFO  4: API   8: DMA  16: IOCTL\n");
+	printf("                     32: I2C   64: IRQ 128: DEC 256: YUV 512: High Volume msgs\n");
 	printf("  -E, --end-gop      capture last GOP [IVTV_IOC_S_GOP_END]\n");
 	printf("  -e, --get-debug    query the module debug variable\n");
 	printf("  -I, --list-gpio\n");
@@ -228,6 +248,10 @@ static void print_debug_mask(int mask)
 		mask &= ~IVTV_DBGFLG_YUV;
 		printf("IVTV_DBGFLG_YUV%s", MASK_OR_NOTHING);
 	}
+	if (mask & IVTV_DBGFLG_HIGHVOL) {
+		mask &= ~IVTV_DBGFLG_HIGHVOL;
+		printf("IVTV_DBGFLG_HIGHVOL%s", MASK_OR_NOTHING);
+	}
 	if (mask) {
 		printf("0x%08x", mask);
 	}
@@ -322,9 +346,9 @@ static void print_regs(int fd, struct v4l2_register *reg, unsigned long min, uns
 		reg->reg = i;
 		if (ioctl(fd, VIDIOC_INT_G_REGISTER, reg) < 0) {
 			fprintf(stderr, "ioctl: VIDIOC_INT_G_REGISTER "
-					"failed for 0x%08lx\n", reg->reg);
+					"failed for 0x%08x\n", (__u32)reg->reg);
 		} else {
-			printf("%0*x ", 2 * stride, reg->val);
+			printf("%0*x ", 2 * stride, (__u32)reg->val);
 		}
 		usleep(1);
 	}
@@ -386,6 +410,10 @@ int main(int argc, char **argv)
 	char scrstr[64];
 	char short_options[26 * 2 * 2 + 1];
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
+	reg.match_type = V4L2_CHIP_MATCH_I2C_DRIVER;
+#endif
+
 	if (argc == 1) {
 		usage();
 		return 0;
@@ -672,10 +700,15 @@ int main(int argc, char **argv)
 	}
 
 	if (options[OptSetRegister]) {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
+		if (reg.i2c_id == 0) {
+			reg.match_type = V4L2_CHIP_MATCH_HOST;
+		}
+#endif
 		if (doioctl(fd, VIDIOC_INT_S_REGISTER, &reg,
 			"VIDIOC_INT_S_REGISTER") == 0)
-			printf("register 0x%08lx set to 0x%08x\n",
-		       		reg.reg, reg.val);
+			printf("register 0x%08x set to 0x%08x\n",
+		       		(__u32)reg.reg, (__u32)reg.val);
 	}
 
 	if (options[OptSetAudioRoute]) {
@@ -685,7 +718,12 @@ int main(int argc, char **argv)
 	if (options[OptListRegisters]) {
 		int stride = 1;
 
-		if (reg.i2c_id == 0) stride = 4;
+		if (reg.i2c_id == 0) {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
+			reg.match_type = V4L2_CHIP_MATCH_HOST;
+#endif
+			stride = 4;
+		}
 		printf("ioctl: VIDIOC_INT_G_REGISTER\n");
 		if (max_reg == 0) {
 			switch (reg.i2c_id) {
@@ -744,32 +782,38 @@ int main(int argc, char **argv)
 	if (options[OptSetGPIO]) {
 		struct v4l2_register reg;
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
+		reg.match_type = V4L2_CHIP_MATCH_HOST;
+#endif
 		reg.i2c_id = 0;
 		reg.reg = IVTV_REG_GPIO_DIR_OFFSET;
 		reg.val = gpio_dir;
 		if (gpio_set_dir && doioctl(fd, VIDIOC_INT_S_REGISTER, &reg,
 			"VIDIOC_INT_S_REGISTER") == 0)
-			printf("GPIO dir set to 0x%04x\n", reg.val);
+			printf("GPIO dir set to 0x%04x\n", (__u32)reg.val);
 		reg.reg = IVTV_REG_GPIO_OUT_OFFSET;
 		reg.val = gpio_out;
 		if (doioctl(fd, VIDIOC_INT_S_REGISTER, &reg,
 			"VIDIOC_INT_S_REGISTER") == 0)
-			printf("GPIO out set to 0x%04x\n", reg.val);
+			printf("GPIO out set to 0x%04x\n", (__u32)reg.val);
 	}
 
 	if (options[OptListGPIO]) {
 		struct v4l2_register reg;
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 20)
+		reg.match_type = V4L2_CHIP_MATCH_HOST;
+#endif
 		reg.i2c_id = 0;
 		reg.reg = IVTV_REG_GPIO_IN_OFFSET;
 		if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) == 0) 
-			printf("GPIO in:  0x%04x\n", reg.val);
+			printf("GPIO in:  0x%04x\n", (__u32)reg.val);
 		reg.reg = IVTV_REG_GPIO_DIR_OFFSET;
 		if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) == 0) 
-			printf("GPIO dir: 0x%04x\n", reg.val);
+			printf("GPIO dir: 0x%04x\n", (__u32)reg.val);
 		reg.reg = IVTV_REG_GPIO_OUT_OFFSET;
 		if (ioctl(fd, VIDIOC_INT_G_REGISTER, &reg) == 0) 
-			printf("GPIO out: 0x%04x\n", reg.val);
+			printf("GPIO out: 0x%04x\n", (__u32)reg.val);
 	}
 
 	if (options[OptSetDebugLevel]) {
@@ -801,7 +845,12 @@ int main(int argc, char **argv)
 	}
 
 	if (option_reset_ir) {
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)
+		__u32 dummy = 0;
+		doioctl(fd, VIDIOC_INT_RESET, &dummy, "VIDIOC_INT_RESET");
+#else
 		doioctl(fd, VIDIOC_INT_RESET, NULL, "VIDIOC_INT_RESET");
+#endif
 	}
 
 	if (option_version) {
